from prediction.endpoints import worker_h2o as endpoints
from prediction import request_utils

def train_model(auth, modelid, modeltype, params, info=False):
	ep = endpoints.BUILD_MODEL
	param_dict = {"model_id": modelid, "model_type": modeltype, "model_parms": params}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp
	# result = resp.json()
	return result

def cancel_job(auth, jobid, info=False):
	ep = endpoints.CANCEL_JOB
	param_dict = {"job_id": jobid}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def delete_frame(auth, frame, info=False):
	ep = endpoints.DELETE_FRAME
	param_dict = {"frame": frame}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	# result = json.resp()
	result = resp
	return result

def download_model_mojo(auth, modelid, info=False):
	ep = endpoints.DOWNLOAD_MODEL_MOJO
	param_dict = {"model_id": modelid}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def get_train_model(auth, modelid, modeltype, info=False):
	ep = None
	if modeltype == "AUTOML":
		ep = endpoints.GET_AUTO_ML_MODEL
	else:
		ep = endpoints.GET_MODEL
	param_dict = {"model_id": modelid}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	try:
		result = resp.json()
		return result
	except:
		return resp

def get_frame(auth, frameid, info=False):
	ep = endpoints.GET_FRAME
	param_dict = {"frame_id": frameid}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	try:
		result = resp.json()
		return result
	except:
		return resp

def get_frame_columns(auth, frameid, info=False):
	ep = endpoints.GET_FRAME_COLUMNS
	param_dict = {"frame_id": frameid}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def get_model_stats(auth, modelid, source, statstype, info=False):
	ep = endpoints.GET_MODEL_STATS
	param_dict = {"model_id": modelid, "source": source, "stats_type": statstype}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def model_grids(auth, model, info=False):
	ep = endpoints.MODEL_GRIDS
	param_dict = {"model": model}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def prediction_frames(auth, info=False):
	ep = endpoints.PREDICTION_FRAMES
	resp = request_utils.create(auth, ep, info=info)
	result = resp.json()
	return result

def prediction_jobs(auth, info=False):
	ep = endpoints.PREDICTION_JOBS
	resp = request_utils.create(auth, ep, info=info)
	result = resp.json()
	return result

def prediction_models(auth, info=False):
	ep = endpoints.PREDICTION_MODELS
	resp = request_utils.create(auth, ep, info=info)
	result = resp.json()
	return result

def file_to_frame(auth, file_name, first_row_column_names, separator, info=False):
	ep = endpoints.PROCESS_FILE_TO_FRAME_IMPORT
	param_dict = {"file_name": file_name, "first_row_column_names": first_row_column_names, "separator": separator}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def featurestore_to_frame(auth, userframe, info=False):
	ep = endpoints.PROCESS_TO_FRAME_PARSE
	resp = request_utils.create(auth, ep, json=userframe, info=info)
	parsed_frame = resp.json()
	return parsed_frame
	
def split_frame(auth, frame, ratio, info=False):
	ep = endpoints.SPLIT_FRAME
	param_dict = {"frame": frame, "ratio": ratio}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def generate_model_detail(auth, info=False):
	ep = endpoints.GENERATE_MODEL_DETAIL
	resp = request_utils.create(auth, ep, info=info)
	result = resp.json()
	return result

def download_model(auth, mojo_id, predict_id, info=False):
	ep = endpoints.DOWNLOAD_MODEL
	param_dict = {"mojo_id": mojo_id, "predict_id": predict_id}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def import_sql_table(auth, json, info=False):
	ep = endpoints.IMPORT_SQL_TABLE
	resp = request_utils.create(auth, ep, json=json, info=info)
	result = resp.json()
	return result

def change_to_enum(auth, frame, column, info=False):
	ep = endpoints.CHANGE_TO_ENUM
	param_dict = {"frame": frame, "column": column}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def get_frame_column_summary(auth, frame, column, info=False):
	ep = endpoints.GET_FRAME_COLUMN_SUMMARY
	param_dict = {"frame": frame, "column": column}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def export_frame(auth, frame, info=False):
	ep = endpoints.EXPORT_FRAME
	param_dict = {"frame": frame}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result




# Data Munging Engine
CONCAT_COLUMNS2 = {
	"type": "get",
	"endpoint": "/2/concatColumns",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
DATE_ENRICH2 = {
	"type": "get",
	"endpoint": "/2/dateEnrich",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
AUTO_NORMALIZE_ALL = {
	"type": "get",
	"endpoint": "/autoNormalizeAll",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
CONCAT_COLUMNS = {
	"type": "get",
	"endpoint": "/concatColumns",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
DATE_ENRICH = {
	"type": "get",
	"endpoint": "/dateEnrich",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
ENUM_CONVERT = {
	"type": "get",
	"endpoint": "/enumConvert",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
FILL_ZEROS = {
	"type": "get",
	"endpoint": "/fillZeros",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
FILL_VALUES = {
	"type": "get",
	"endpoint": "/fillValues",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
FOREIGN_KEY_AGGREGATOR = {
	"type": "get",
	"endpoint": "/foreignKeyAggregator",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
FOREIGN_KEY_LOOKUP = {
	"type": "get",
	"endpoint": "/foreignKeyLookup",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
FRAGMENT_ENRICH = {
	"type": "get",
	"endpoint": "/fragmentEnrich",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
FRAGMENT_ENRICH2 = {
	"type": "get",
	"endpoint": "/fragmentEnrich2",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GENERATE_FEATURES = {
	"type": "get",
	"endpoint": "/generateFeatures",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GENERATE_FEATURES_NORMALIZE = {
	"type": "get",
	"endpoint": "/generateFeaturesNormalize",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_CATEGORIES = {
	"type": "get",
	"endpoint": "/getCategories",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_CATEGORIES_RATIOS = {
	"type": "get",
	"endpoint": "/getCategoriesRatios",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
LOCATION_ENRICH = {
	"type": "get",
	"endpoint": "/locationEnrich",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
MCC_ENRICH = {
	"type": "get",
	"endpoint": "/mccEnrich",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PREDICTION_ENRICH = {
	"type": "get",
	"endpoint": "/predictionEnrich",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PREDICTION_ENRICH_FAST_GET = {
	"type": "get",
	"endpoint": "/predictionEnrichFast",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
SIC_ENRICH = {
	"type": "get",
	"endpoint": "/sicEnrich",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PROCESS_CLIENT_PULSE_RELIABILITY = {
	"type": "get",
	"endpoint": "/processClientPulseReliability",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GENERATE_TIME_SERIES_FEATURES = {
	"type": "get",
	"endpoint": "/generateTimeSeriesFeatures",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

PERSONALITY_ENRICH = {
	"type": "get",
	"endpoint": "/personalityEnrich",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}

MUNGE_TRANSACTIONS_AGGREGATE = {
	"type": "get",
	"endpoint": "/mungeTransactionsAggregate",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}

MUNGE_TRANSACTIONS = {
	"type": "get",
	"endpoint": "/mungeTransactions",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}

FLATTEN_DOCUMENT = {
	"type": "get",
	"endpoint": "/flattenDocument",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}

DELETE_KEY = {
	"type": "get",
	"endpoint": "/deleteKey",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}

NLP_WORKER = {
	"type": "get",
	"endpoint": "/nlpWorker",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}

DELETE_MANY_DOCUMENTS = {
	"type": "get",
	"endpoint": "/deleteManyDocuments",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}

PROCESS_RANGE = {
	"type": "get",
	"endpoint": "/processRange",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}

PREDICTION_ENRICH_FAST_POST = {
	"type": "post",
	"endpoint": "/predictionEnrichFast",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}


from prediction.endpoints import prediction_engine as endpoints
from prediction import request_utils

def delete_analysis(auth, analysis_id, info=False):
	ep = endpoints.DELETE_ANALYSIS
	resp = request_utils.create(auth, ep, ep_arg=analysis_id, info=info)
	result = resp.json()
	return result

def delete_model(auth, model_id, info=False):
	ep = endpoints.DELETE_MODEL
	resp = request_utils.create(auth, ep, ep_arg=model_id, info=info)
	result = resp.json()
	return result

def delete_prediction(auth, prediction_id, info=False):
	ep = endpoints.DELETE_PREDICTION
	resp = request_utils.create(auth, ep, ep_arg=prediction_id, info=info)
	result = resp.json()
	return result

def delete_prediction_project(auth, prediction_project_id, info=False):
	ep = endpoints.DELETE_PREDICTION_PROJECT
	resp = request_utils.create(auth, ep, ep_arg=prediction_project_id, info=info)
	result = resp.json()
	return result

def delete_user_deployments(auth, user_deployments_id, info=False):
	ep = endpoints.DELETE_USER_DEPLOYMENTS
	resp = request_utils.create(auth, ep, ep_arg=user_deployments_id, info=info)
	result = resp.json()
	return result

def delete_userframe(auth, frame_id, info=False):
	ep = endpoints.DELETE_USER_FRAME
	param_dict = {"frame_id": frame_id}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def deploy_predictor(auth, json, info=False):
	ep = endpoints.DEPLOY_PREDICTOR
	resp = request_utils.create(auth, ep, json=json, info=info)
	response = resp.json()
	return response

def download_project(auth, project_id, module_name, info=False):
	ep = endpoints.DOWNLOAD_PROJECT_ZIP
	param_dict = {"projectID": project_id, "moduleName": module_name}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	response = resp
	return response

def get_analysis_result(auth, analysis_id, info=False):
	ep = endpoints.GET_ANALYSIS_RESULT
	param_dict = {"analysis_id": analysis_id}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def get_analysis_results(auth, info=False):
	ep = endpoints.GET_ANALYSIS_RESULTS
	param_dict = {"user": auth.get_username()}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	results = resp.json()
	if "item" in results:
		results = results["item"]
	return results

def get_prediction(auth, predict_id, info=False):
	ep = endpoints.GET_PREDICTION
	param_dict = {"predict_id": predict_id}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def get_prediction_project(auth, project_id, info=False):
	ep = endpoints.GET_PREDICTION_PROJECT
	param_dict = {"project_id": project_id}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

# TODO: all these ep.get_username() call might need to be replaced with arguments that check if a  value is passed first
def get_prediction_projects(auth, info=False):
	ep = endpoints.GET_PREDICTION_PROJECTS
	param_dict = {"user": auth.get_username()}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	projects = resp.json()
	if "item" in projects:
		projects = projects["item"]
	return projects

def get_user_deployment(auth, user_deployments, info=False):
	ep = endpoints.GET_USER_DEPLOYMENT
	param_dict = {"user_deployments": user_deployments}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def get_user_deployments(auth, info=False):
	ep = endpoints.GET_USER_DEPLOYMENTS
	param_dict = {"user": auth.get_username()}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	deployments = resp.json()
	if "item" in deployments:
		deployments = deployments["item"]
	return deployments

def get_user_featurestores(auth, info=False):
	ep = endpoints.GET_USER_FEATURE_STORES
	param_dict = {"user": auth.get_username()}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	featurestores = resp.json()
	if "item" in featurestores:
		featurestores = featurestores["item"]
	return featurestores

def get_user_files(auth, info=False):
	ep = endpoints.GET_USER_FILES
	param_dict = {"user": auth.get_username()}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	files = resp.json()
	if "item" in files:
		files = files["item"]
	return files

def get_featurestores(auth, info=False):
	ep = endpoints.GET_USER_FRAMES
	param_dict = {"user": auth.get_username()}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	frames = resp.json()
	if "item" in frames:
		frames = frames["item"]
	return frames

def get_featurestore(auth, frame_id, info=False):
	ep = endpoints.GET_USER_FRAME
	param_dict = {"frame_id": frame_id}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	userframe = resp.json()
	return userframe

def get_uframe(auth, frame_id, info=False):
	ep = endpoints.GET_USER_FRAME
	param_dict = {"frame_id": frame_id}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp
	return result

def get_user_model(auth, model_identity, info=False):
	ep = endpoints.GET_USER_MODEL
	param_dict = {"model_identity": model_identity}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def get_user_models(auth, info=False):
	ep = endpoints.GET_USER_MODELS
	param_dict = {"user": auth.get_username()}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	models = resp.json()
	if "item" in models:
		models = models["item"]
	return models

def get_user_predictions(auth, info=False):
	ep = endpoints.GET_USER_PREDICTIONS
	param_dict = {"user": auth.get_username()}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	predictions = resp.json()
	if "item" in predictions:
		predictions = predictions["item"]
	return predictions

def import_module(auth, module_id, info=False):
	ep = endpoints.IMPORT_MODULE
	param_dict = {"module_id": module_id}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def save_analysis(auth, analysis, info=False):
	ep = endpoints.SAVE_ANALYSIS
	resp = request_utils.create(auth, ep, json=analysis, info=info)
	response = resp.json()
	return response

def save_model(auth, model, info=False):
	ep = endpoints.SAVE_MODEL
	resp = request_utils.create(auth, ep, json=model, info=info)
	response = resp.json()
	return response

def save_prediction(auth, prediction, info=False):
	ep = endpoints.SAVE_PREDICTION
	resp = request_utils.create(auth, ep, json=prediction, info=info)
	response = resp.json()
	return response

def save_analysis(auth, analysis, info=False):
	ep = endpoints.SAVE_ANALYSIS
	resp = request_utils.create(auth, ep, json=analysis, info=info)
	response = resp.json()
	return response

def save_prediction_project(auth, prediction_project, info=False):
	ep = endpoints.SAVE_PREDICTION_PROJECT
	resp = request_utils.create(auth, ep, json=prediction_project, info=info)
	response = resp.json()
	return response

def save_user_deployments(auth, user_deployments, info=False):
	ep = endpoints.SAVE_USER_DEPLOYMENTS
	resp = request_utils.create(auth, ep, json=user_deployments, info=info)
	response = resp.json()
	return response

def save_user_frame(auth, user_frame, info=False):
	ep = endpoints.SAVE_USER_FRAME
	resp = request_utils.create(auth, ep, json=user_frame, info=info)
	response = resp.json()
	return response

def test_model(auth, value, info=False):
	ep = endpoints.TEST_PREDICTOR
	param_dict = {"value": value}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	response = resp.json()
	return response

def get_prediction_projects_base(auth, info=False):
	ep = endpoints.GET_PREDICTION_PROJECTS_BASE
	param_dict = {"user": auth.get_username()}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	response = resp.json()
	return response


import requests
from prediction.endpoints import auth_controller as endpoints
from datetime import datetime
import time

def stringify_data(data):
	str_data = str(data).replace("'", '"') 	# API requires input data to be in string format and must use " for denoting key and value pairs. Python's
											# to string method uses ' as to indicate strings in the key value pairs.
	return str_data

class AuthenticationError(Exception):
	"""Exception raised for incorrect authentication details."""
	def __init__(self):
		message = "Username and Password combination not valid."
		super().__init__(message)

class AuthenticationExpiry(Exception):
	"""Exception raised for expired authentication token."""
	def __init__(self, expiry):
		expiry_datetime = datetime.utcfromtimestamp(expiry).strftime("%Y-%m-%d %H:%M:%S")
		message = "Authentication token expired on {}. Please login again.".format(expiry_datetime)
		super().__init__(message)

class Authenticate:
	def __init__(self, api_address, username, password):
		self.api_address = api_address
		self.username = username
		self.password = password
		token = self.__login(api_address, username, password)
		self.access_token = token["access_token"]
		self.refresh_token = token["refresh_token"]
		self.expires_in = self.__format_time(token["expires_in"])

		self.auth_headers = {
			"Accept": "*/*",
			"Authorization": "Bearer {}".format(self.access_token)
		}

	def __login(self, api_address, username, password):
		header = {
			"Accept": "*/*",
			"Content-Type": "application/json"
		}

		data = {
			"email": username,
			"password": password
		}
		
		str_data = stringify_data(data) # Stringify dict: data need to be in string format
		r = requests.post("{}{}".format(api_address, endpoints.AUTH_LOGIN["endpoint"]), headers=header, data=str_data)
		if r.status_code == 403:
			raise AuthenticationError()
		token = r.json()["token"]
		print("Login Successful.")
		return token

	def __format_time(self, t):
		# unix time in jwt authentication process seems to be oddly formated.
		# this function aligns the jwt expiry time with the python unix time.
		front = str(t)[:-3]
		end = str(t)[-3:]
		return float(front + "." + end)

	def __token_expired(self):
		current_time = time.time()
		if current_time >= self.expires_in:
			raise AuthenticationExpiry(self.expires_in)
		return False

	def get_username(self):
		if not self.__token_expired():
			return self.username

	def get_password(self):
		if not self.__token_expired():
			return self.password

	def get_server(self):
		if not self.__token_expired():
			return self.api_address

	def get_access_token(self):
		if not self.__token_expired():
			return self.access_token

	def get_refresh_token(self):
		if not self.__token_expired():
			return self.refresh_token

	def get_expiry_date(self):
		if not self.__token_expired():
			return self.expires_in

	def get_auth_headers(self):
		if not self.__token_expired():
			return self.auth_headers


from prediction.endpoints import algorithm_client_pulse as endpoints
from prediction import request_utils

def get_timeline(auth, user, limit, info=False):
# Get Timeline:
#   auth: Authentication token generated by jwt_access.Authenticate()
#   user: User name (string)
#   limit: Output limit (int)
	ep = endpoints.GET_TIMELINE
	param_dict = {
		"user": user, 
		"limit": limit
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def process_basket(auth, dbcust, colcust, searchcust, custfield, dbitem, colitem, itemfield, supportcount, info=False):
# Process Basket:
#   auth: Authentication token generated by jwt_access.Authenticate()
#   dbcust: (str)
#   colcust: (str)
#   searchcust: (str)
#   custfield: (str)
#   dbitem: (str)
#   colitem: (str)
#   itemfield: (str)
#   supportcount: (int)
	ep = endpoints.PROCESS_BASKET
	param_dict = {
		"dbCust": dbcust, 
		"colCust": colcust,
		"searchCust": searchcust,
		"custField": custfield,
		"dbItem": dbitem,
		"colItem": colitem,
		"itemField": itemfield,
		"supportCount": supportcount
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def process_directed_graph(auth, graphMeta, graphParam, info=False):
# Analyze graph from data using graph meta data and analysis parameters. 
# Meta data format example: {"vertex":[0,1],"edges":[{"from":0,"to":1}],"from":0,"source":"/data/data.csv","dotfile":"/data/data.dot","to":1} Analysis parameter example: {"destination":"Bakeries","source":"12345678"} 
# Columns in file start at 0 and is sequenced.
#   auth: Authentication token generated by jwt_access.Authenticate()
#   graphMeta: (str)
#   graphParam: (str)
	ep = endpoints.PROCESS_BASKET
	param_dict = {
		"graphMeta": graphMeta, 
		"graphParam": graphParam
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta


def generate_forecast(auth, attribute, collection, collectionOut, database, dateattribute, find, historicsteps, steps, info=False):
	ep = endpoints.GENERATE_FORECAST
	param_dict = {
		"attribute": attribute,
		"collection": collection,
		"collectionOut": collectionOut,
		"database": database,
		"dateattribute": dateattribute,
		"find": find,
		"historicsteps": historicsteps,
		"steps": steps
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def process_apriori(auth, colItem, collection, collectionOut, custField, database, dbItem, find, itemField, supportCount, info=False):
	ep = endpoints.PROCESS_APRIORI
	param_dict = {
		"colItem": colItem,
		"collection": collection,
		"collectionOut": collectionOut,
		"custField": custField,
		"database": database,
		"dbItem": dbItem,
		"find": find,
		"itemField": itemField,
		"supportCount": supportCount
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def process_ecogenetic_network(auth, collection, collectionOut, database, find, graphMeta, graphParam, info=False):
	ep = endpoints.PROCESS_ECOGENETIC_NETWORK
	param_dict = {
		"collection": collection,
		"collectionOut": collectionOut,
		"database": database,
		"find": find,
		"graphMeta": graphMeta,
		"graphParam": graphParam
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def process_directed_graph2(auth, db, collection, collection_out, graphMeta, graphParam, info=False):
# Analyze graph from data using graph meta data and analysis parameters. 
# Meta data format example: {"vertex":[0,1],"edges":[{"from":0,"to":1}],"from":0,"source":"/data/data.csv","dotfile":"/data/data.dot","to":1} Analysis parameter example: {"destination":"Bakeries","source":"12345678"} 
# Columns in file start at 0 and is sequenced.
#   auth: Authentication token generated by jwt_access.Authenticate()
#   graphMeta: (str)
#   graphParam: (str)
	ep = endpoints.PROCESS_DIRECTED_GRAPH2
	param_dict = {
		"database": db,
		"collection": collection,
		"collectionOut": collection_out,
		"graphMeta": graphMeta,
		"graphParam": graphParam
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def generate_cox_ph_data(auth, collection, collection_out, customer_field, database, database_out, date_field, find, info=False):
	ep = endpoints.GENERATE_COX_PH_DATA
	param_dict = {
		"collection": collection, 
		"collectionOut": collection_out,
		"customer_field": customer_field,
		"database": database,
		"databaseOut": database_out,
		"date_field": date_field,
		"find": find
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def process_arima_forecast(auth, json, info=False):
	ep = endpoints.PROCESS_ARIMA_FORECAST
	resp = request_utils.create(auth, ep, json=json, info=info)
	response = resp.json()
	return response



def list_pulse_responder_dynamic(auth, info=False):
	ep = endpoints.LIST_PULSE_RESPONDER_DYNAMIC
	resp = request_utils.create(auth, ep, info=info)
	result = resp.json()
	return result

def delete_pulse_responder_dynamic(auth, db, collection, find, info=False):
	ep = endpoints.DELETE_PULSE_RESPONDER_DYNAMIC
	param_dict = {
		"database": db,
		"collection": collection,
		"find": find,
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def get_pulse_responder_messages(auth, db, collection, info=False):
	ep = endpoints.GET_PULSE_RESPONDER_MESSAGES
	param_dict = {
		"database": db,
		"collection": collection,
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def get_pulse_responder_options(auth, params, info=False):
	ep = endpoints.GET_PULSE_RESPONDER_OPTIONS
	param_dict = {
		"params": params,
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def get_pulse_responder_profile(auth, db, collection, info=False):
	ep = endpoints.GET_PULSE_RESPONDER_PROFILE
	param_dict = {
		"database": db,
		"collection": collection,
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	result = resp.json()
	return result

def update_client_pulse_responder(auth, json, info=False):
	ep = endpoints.UPDATE_CLIENT_PULSE_RESPONDER
	resp = request_utils.create(auth, ep, json=json, info=info)
	response = resp.json()
	return response

def save_pulse_responder_dynamic(auth, json, info=False):
	ep = endpoints.SAVE_PULSE_RESPONDER_DYNAMIC
	resp = request_utils.create(auth, ep, json=json, info=info)
	response = resp.json()
	return response


from prediction.endpoints import ecosystem_generation_engine as endpoints
from prediction import request_utils

def generate_build(auth, json, info=False):
	ep = endpoints.GENERATE_BUILD
	resp = request_utils.create(auth, ep, json=json, info=False)
	result = resp.json()
	return result

# def generate_properties(auth, json, info=False):
# 	ep = endpoints.GENERATE_PROPERTIES
# 	resp = request_utils.create(auth, ep, json=json, info=False)
# 	result = resp.json()
# 	return result

def get_build(auth, uuid, info=False):
	ep = endpoints.GET_BUILD
	param_dict = {
		"uuid": uuid
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=False)
	meta = resp.json()
	if "data" in meta:
		meta = meta["data"]
	return meta

def process_build(auth, json, info=False):
	ep = endpoints.PROCESS_BUILD
	resp = request_utils.create(auth, ep, json=json, info=False)
	result = resp.json()
	return result

def process_push(auth, json, info=False):
	ep = endpoints.PROCESS_PUSH
	resp = request_utils.create(auth, ep, json=json, info=False)
	result = resp.json()
	return result


# Data Ingestion Engine
ADD_META_DOCUMENTS = {
	"type": "post",
	"endpoint": "/addMetaDocuments",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
DELETE_INGEST_META = {
	"type": "post",
	"endpoint": "/deleteIngestMeta",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

GET_DATABASES_META = {
	"type": "get",
	"endpoint": "/getDatabasesMeta",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_DATABASE_TABLE_COLUMN_META = {
	"type": "get",
	"endpoint": "/getDatabaseTableColumnMeta",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_DATABASE_TABLE_COLUMNS_META = {
	"type": "get",
	"endpoint": "/getDatabaseTableColumnsMeta",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_DATABASE_TABLES_META = {
	"type": "get",
	"endpoint": "/getDatabaseTablesMeta",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_INGEST_META = {
	"type": "get",
	"endpoint": "/getIngestMeta",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_INGEST_METAS = {
	"type": "get",
	"endpoint": "/getIngestMetas",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
SAVE_DATABASE_TABLE_COLUMN_META = {
	"type": "post",
	"endpoint": "/saveDatabaseTableColumnMeta",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
SAVE_INGEST_META = {
	"type": "post",
	"endpoint": "/saveIngestMeta",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}


from prediction.endpoints import data_ingestion_engine as endpoints
from prediction import request_utils

def add_metadocumemnts(auth, meta_documents, info=False):
    ep = endpoints.ADD_META_DOCUMENTS
    resp = request_utils.create(auth, ep, json=meta_documents, info=info)
    response = resp.json()
    return response

def delete_ingestmeta(auth, ingest_meta, info=False):
    ep = endpoints.DELETE_INGEST_META
    resp = request_utils.create(auth, ep, json=ingest_meta, info=info)
    response = resp.json()
    return response	

def get_databasesmeta(auth, info=False):
    ep = endpoints.GET_DATABASES_META
    resp = request_utils.create(auth, ep, info=info)
    meta = resp.json()
    if "data" in meta:
        meta = meta["data"]
    return meta

def get_databasetablecolumnmeta(auth, databasename, tablename, columnname, info=False):
    ep = endpoints.GET_DATABASE_TABLE_COLUMN_META
    param_dict = {
        "databasename": databasename,
        "tablename": tablename,
        "columnname": columnname
    }
    resp = request_utils.create(auth, ep, params=param_dict, info=info)
    meta = resp.json()
    if "data" in meta:
        meta = meta["data"]
    return meta

def get_databasetablecolumnsmeta(auth, databasename, tablename, info=False):
    ep = endpoints.GET_DATABASE_TABLE_COLUMNS_META
    param_dict = {"databasename": databasename, "tablename": tablename}
    resp = request_utils.create(auth, ep, params=param_dict, info=info)
    meta = resp.json()
    if "data" in meta:
        meta = meta["data"]
    return meta

def get_databasetablesmeta(auth, databasename, info=False):
    ep = endpoints.GET_DATABASES_TABLES_META
    param_dict = {"databasename": databasename}
    resp = request_utils.create(auth, ep, params=param_dict, info=info)
    meta = resp.json()
    if "data" in meta:
        meta = meta["data"]
    return meta

def get_ingestmeta(auth, ingest_name, info=False):
    ep = endpoints.GET_INGEST_META
    param_dict = {"ingest_name": ingest_name}
    resp = request_utils.create(auth, ep, params=param_dict, info=info)
    meta = resp.json()
    if "data" in meta:
        meta = meta["data"]
    return meta

def get_ingestmetas(auth, info=False):
    ep = endpoints.GET_INGEST_METAS
    resp = request_utils.create(auth, ep, info=info)
    meta = resp.json()
    if "data" in meta:
        meta = meta["data"]
    return meta

def save_databasetablecolumn(auth, database_table_column_json, info=False):
    ep = endpoints.SAVE_DATABASE_TABLE_COLUMN_META
    resp = request_utils.create(auth, ep, json=database_table_column_json, info=info)
    response = resp.json()
    return response

def save_ingestion(auth, ingestion_json, info=False):
    ep = endpoints.SAVE_INGEST_META
    resp = request_utils.create(auth, ep, json=ingestion_json, info=info)
    response = resp.json()
    return response


import datetime
from prediction.apis import worker_h2o
from prediction.apis import prediction_engine
import json

def save_file_as_userframe(auth, data_file, feature_store, user_name):
	timestampvar = datetime.datetime.now().isoformat()
	data_file_prefix = ".".join(data_file.split(".")[:-1])
	hexframename = data_file_prefix + ".hex"
	imp = worker_h2o.file_to_frame(auth, data_file, 1, ",")
	# ['import', 'parseSetup', 'columnNames', 'columnTypes', 'columnProperties'
	descrip = "Automated feature store generated for " + feature_store
	frameID = feature_store
	str_imp = {
		"import": imp["import"],
		"parseSetup": imp["parseSetup"],
		"columnNames": imp["columnNames"],
		"columnTypes": imp["columnTypes"],
		"columnProperties": imp["columnProperties"]
	}
	str_imp = json.dumps(str_imp)
	# str_imp = str(str_imp).replace("'", '"').replace("None", "null").replace("False", "false").replace("True", "true")
	user_frame = {
		"timestamp_parsed": timestampvar,
		"parser": "csv",
		"import": str_imp,
		"destination_frame": hexframename,
		"file_name": data_file,
		"description": descrip,
		"first_row_column_names": "1",
		"feature_store_script": "",
		"separator": "comma",
		"frame_id": frameID,
		"columnProperties": imp["columnProperties"],
		"created_by": user_name,
		"created_date": timestampvar,
		"preview_detail": {
			"summary": descrip,
			"image": "/data/xyz.png",
			"heading": frameID,
			"active": True,
			"detail": data_file
		}
	}

	worker_h2o.delete_frame(auth, hexframename)
	prediction_engine.save_user_frame(auth, user_frame)
	frame = worker_h2o.featurestore_to_frame(auth, user_frame)
	return frame["parseSetup"]["destination_frame"], imp

def get_list_of_fields(db, collection):
	fields_pipeline = [
		{"$project":{"arrayofkeyvalue":{"$objectToArray":"$$ROOT"}}},
		{"$unwind":"$arrayofkeyvalue"},
		{"$group":{"_id":None,"allkeys":{"$addToSet":"$arrayofkeyvalue.k"}}}
	]
	cursor = db[collection].aggregate(fields_pipeline)
	doc = cursor.next()
	list_of_fields = doc["allkeys"]
	if "_id" in list_of_fields: list_of_fields.remove("_id")
	return list_of_fields


SET_CREDENTIALS = {
	"type": "get",
	"endpoint": "/setCredentials",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

POST_SET_CREDENTIALS = {
	"type": "post",
	"endpoint": "/setCredentials",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}


SET_TENANT = {
	"type": "get",
	"endpoint": "/setTenant",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

MANAGE_TENANT = {
	"type": "get",
	"endpoint": "/manageTenant",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

CREATE_TENANT = {
	"type": "get",
	"endpoint": "/createTenant",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

POST_CREATE_TENANT = {
	"type": "post",
	"endpoint": "/createTenant",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

DELETE_TENANT = {
	"type": "get",
	"endpoint": "/deleteTenant",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

SET_CONFIGURATION = {
	"type": "get",
	"endpoint": "/setConfiguration",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

VALIDATE_TENANT = {
	"type": "get",
	"endpoint": "/validateTenant",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

GET_STATUS = {
	"type": "get",
	"endpoint": "/getStatus",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

LIST_TENANTS = {
	"type": "get",
	"endpoint": "/listTenants",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}


LOGIN = {
	"type": "get",
	"endpoint": "/login",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PING = {
	"type": "get",
	"endpoint": "/ping",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}


from prediction.apis import algorithm_client_pulse as cp
from prediction.apis import data_management_engine as d

from datetime import datetime
import json
import uuid

def online_learning_ecosystem_rewards_setup_feature_store(
    auth,
    offer_db,
    offer_collection,
    offer_name_column,
    contextual_variables,
    setup_feature_store_db,
    setup_feature_store_collection
):
    if not isinstance(offer_db, str):
        raise TypeError("offer_db should be a string")
    if not isinstance(offer_collection, str):
        raise TypeError("offer_collection should be a string")
    if not isinstance(offer_name_column, str):
        raise TypeError("offer_name_column should be a string")
    if not isinstance(contextual_variables, dict):
        raise TypeError("contextual_variables should be a dictionary")
    if not isinstance(setup_feature_store_db, str):
        raise TypeError("setup_feature_store_db should be a string")
    if not isinstance(setup_feature_store_collection, str):
        raise TypeError("setup_feature_store_collection should be a string")

    try:
        sample_offer_collection = d.get_data(auth, offer_db, offer_collection, {}, 1, {}, 0)
        if len(sample_offer_collection) == 0:
            raise ImportError(f"Mongo collection {offer_collection} in database {offer_db} appears to be empty")
        if offer_name_column not in sample_offer_collection:
            raise KeyError(f"{offer_name_column}, specified as the offer_name_colume, not found as a field in {offer_collection}")
    except Exception as error:
        print("Unexpected error occured while validating offer_db and offer_collection")
        raise
    
    if len(contextual_variables) > 2:
        raise KeyError("At most two contextual variables can be specified for the ecosystem rewards algorithm")
    for context_var_iter in contextual_variables:
        if not isinstance(contextual_variables[context_var_iter], list):
            raise TypeError("The value for each key in the contextual variables dictionary should be a list of possible segment values")    

    contextual_fields = list(contextual_variables.keys())
    if len(contextual_fields) == 2:
        d.post_mongo_db_aggregate_pipeline(auth, 
            {
            "database":offer_db
            ,"collection":offer_collection
            ,"pipeline":[
                {"$group":{"_id":"$"+offer_name_column}}
                ,{"$project":{offer_name_column:"$_id","_id":0}}
                ,{"$addFields":{contextual_fields[0]:contextual_variables[contextual_fields[0]]}}
                ,{"$unwind":"$"+contextual_fields[0]}
                ,{"$addFields":{contextual_fields[1]:contextual_variables[contextual_fields[1]]}}
                ,{"$unset":"_id"}
                ,{"$unwind":"$"+contextual_fields[1]}
                ,{"$out":{"db":setup_feature_store_db,"coll":setup_feature_store_collection}}
            ]
            }
        )
    elif len(contextual_fields) == 1:
        d.post_mongo_db_aggregate_pipeline(auth, 
            {
            "database":offer_db
            ,"collection":offer_collection
            ,"pipeline":[
                {"$group":{"_id":"$"+offer_name_column}}
                ,{"$project":{offer_name_column:"$_id","_id":0}}
                ,{"$addFields":{contextual_fields[0]:contextual_variables[contextual_fields[0]]}}
                ,{"$unwind":"$"+contextual_fields[0]}
                ,{"$out":{"db":setup_feature_store_db,"coll":setup_feature_store_collection}}
            ]
            }
        )

def create_online_learning(
        auth,
        name,
        description,
        feature_store_collection,
        feature_store_database,
        options_store_database,
        options_store_collection,
        contextual_variables_offer_key,
        score_connection = "http://ecosystem-runtime:8091",
        score_database = "ecosystem_meta",
        score_collection = "dynamic_engagement",
        algorithm = "ecosystem_rewards",
        options_store_connection = "",
        batch = "false",       
        feature_store_connection = "",
        contextual_variables_contextual_variable_one_from_data_source = False,
        contextual_variables_contextual_variable_one_lookup = "",
        contextual_variables_contextual_variable_one_name = "",
        contextual_variables_contextual_variable_two_from_data_source = False,
        contextual_variables_contextual_variable_two_name = "",
        contextual_variables_contextual_variable_two_lookup = "",
        contextual_variables_tracking_key = "",
        contextual_variables_take_up = "",
        batch_database_out = "",
        batch_collection_out = "",
        batch_threads = 1,
        batch_collection = "",
        batch_userid = "",
        batch_contextual_variables = "",
        batch_number_of_offers = 1,
        batch_database = "",
        batch_pulse_responder_list = "",
        batch_find = "{}",
        batch_options = "",
        batch_campaign = "",
        batch_execution_type = "",
        randomisation_calendar = "None",
        randomisation_test_options_across_segment = "",
        randomisation_processing_count = 1000,
        randomisation_discount_factor = 0.75,
        randomisation_batch = "false",
        randomisation_prior_fail_reward = 0.1,
        randomisation_cross_segment_epsilon = 0,
        randomisation_success_reward = 1,
        randomisation_interaction_count = "0",
        randomisation_epsilon = 0,
        randomisation_prior_success_reward = 1,
        randomisation_fail_reward = 0.1,
        randomisation_max_reward = 10,
        randomisation_cache_duration = 0,
        randomisation_processing_window = 86400000,
        randomisation_random_action = 0.2,
        randomisation_decay_gamma = "1",
        randomisation_learning_rate = 0.25,
        replace = False
):
    #TODO add error checking and handling
    #Check for existence of online learning configuration with the same name
    existing_configurations = cp.list_pulse_responder_dynamic(auth)
    if not replace:
        if len([d for d in existing_configurations["data"] if d["name"] == name]) > 0:
            raise ValueError(f"There is an existing online learning configuration named {name} and replace is False")      
    
    #Initialise configuration document and store input parameters
    config_doc = {}
    config_doc["name"] = name
    config_doc["description"] = description
    
    config_doc["feature_store_collection"] = feature_store_collection
    config_doc["feature_store_database"] = feature_store_database
    config_doc["feature_store_connection"] = feature_store_connection
    
    config_doc["options_store_database"] = options_store_database
    config_doc["options_store_collection"] = options_store_collection
    config_doc["options_store_connection"] = options_store_connection
    
    config_doc["score_collection"] = score_collection
    config_doc["score_database"] = score_database
    config_doc["score_connection"] = score_connection
    
    config_doc["batch"] = batch
    config_doc["options"] = []
    config_doc["date_updated"] = datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%f")
    config_doc["description"] = description
    config_doc["uuid"] = str(uuid.uuid4())
    
    #Get values for contextual variables
    if contextual_variables_contextual_variable_one_name != "":
        contextual_variable_one_values_pipeline = [
            {"$group":{"_id":"None","values":{"$addToSet":"$"+contextual_variables_contextual_variable_one_name}}}
        ]
        contextual_variable_one_values = d.post_mongo_db_aggregate_pipeline(
                auth,
                {"database": feature_store_database, "collection": feature_store_collection, "pipeline": contextual_variable_one_values_pipeline}
        )[0]["values"]
    else:
        contextual_variable_one_values = []
    
    if contextual_variables_contextual_variable_two_name != "":
        contextual_variable_two_values_pipeline = [
            {"$group":{"_id":"None","values":{"$addToSet":"$"+contextual_variables_contextual_variable_two_name}}}
        ]
        contextual_variable_two_values = d.post_mongo_db_aggregate_pipeline(
                auth,
                {"database": feature_store_database, "collection": feature_store_collection, "pipeline": contextual_variable_two_values_pipeline}
        )[0]["values"]
    else:
        contextual_variable_two_values = []
    
    contextual_variables = {
        "offer_key": contextual_variables_offer_key,
        "offer_values": [],
        "contextual_variable_one_from_data_source": contextual_variables_contextual_variable_one_from_data_source,
        "contextual_variable_one_lookup": contextual_variables_contextual_variable_one_lookup,
        "contextual_variable_one_name": contextual_variables_contextual_variable_one_name,
        "contextual_variable_one_values": contextual_variable_one_values,
        "contextual_variable_two_from_data_source": contextual_variables_contextual_variable_two_from_data_source,
        "contextual_variable_two_name": contextual_variables_contextual_variable_two_name,
        "contextual_variable_two_lookup": contextual_variables_contextual_variable_two_lookup,
        "contextual_variable_two_values": contextual_variable_two_values,
        "tracking_key": contextual_variables_tracking_key,
        "take_up": contextual_variables_take_up
    }    
    config_doc["contextual_variables"] = contextual_variables
    
    batch_settings = {
        "database_out": batch_database_out,
        "collection_out": batch_collection_out,
        "batchUpdateMessage": "",
        "threads": batch_threads,
        "collection": batch_collection,
        "userid": batch_userid,
        "contextual_variables": batch_contextual_variables,
        "number_of_offers": batch_number_of_offers,
        "batch_outline": "",
        "pulse_responder_list": batch_pulse_responder_list,
        "database": batch_database,
        "find": batch_find,
        "options": batch_options,
        "campaign": batch_campaign,
        "execution_type": batch_execution_type
    }
    config_doc["batch_settings"] = batch_settings
    
    if algorithm not in ["ecosystem_rewards"]:
        raise ValueError("algorithm must be ecosystem_rewards algorithm for other algorithms please use the ecosystem.Ai workbench")
    else:
        randomisation = {
            "calendar": randomisation_calendar,
            "test_options_across_segment": randomisation_test_options_across_segment,
            "processing_count": randomisation_processing_count,
            "discount_factor": randomisation_discount_factor,
            "batch": randomisation_batch,
            "prior_fail_reward": randomisation_prior_fail_reward,
            "approach": "binaryThompson",
            "cross_segment_epsilon": randomisation_cross_segment_epsilon,
            "success_reward": randomisation_success_reward,
            "interaction_count": randomisation_interaction_count,
            "epsilon": randomisation_epsilon,
            "prior_success_reward": randomisation_prior_success_reward,
            "fail_reward": randomisation_fail_reward,
            "max_reward": randomisation_max_reward,
            "cache_duration": randomisation_cache_duration,
            "processing_window": randomisation_processing_window,
            "random_action": randomisation_random_action,
            "decay_gamma": randomisation_decay_gamma,
            "learning_rate": randomisation_learning_rate
        }
        config_doc["randomisation"] = randomisation
  
    config_doc["lookup_fields"] = []
    config_doc["virtual_variables"] = []
    
    properties_list = [
        {"uuid":config_doc["uuid"], "type":"dynamic_engagement", "name":"dynamic_engagement", "database":"mongodb", "db":config_doc["score_database"], "table":config_doc["score_collection"], "update":True}
        ,{"uuid":config_doc["uuid"], "type":"dynamic_engagement_options", "name":"dynamic_engagement", "database":"mongodb", "db":config_doc["options_store_database"], "table":config_doc["options_store_collection"], "update":True}
    ]
    config_doc["properties"] = json.dumps(properties_list)
    
    if len([d for d in existing_configurations["data"] if d["name"] == name]) > 0:
        cp.delete_pulse_responder_dynamic(auth,score_database,score_collection,{"name":name})
        
    d.add_documents(auth, {"database": config_doc["score_database"], "collection": config_doc["score_collection"], "document": config_doc})
    
    doc_delete = {
        "database": config_doc["options_store_database"]
        ,"collection": config_doc["options_store_collection"]
    }
    d.delete_all_documents(auth, doc_delete)
    cp_update_doc = {
        "type":"generateDefaultOptions"
        ,"name":config_doc["name"]
        ,"uuid":config_doc["uuid"]
        ,"engagement_type":"binaryThompson"
        ,"feature_store_database":config_doc["feature_store_database"]
        ,"feature_store_collection":config_doc["feature_store_collection"]
        ,"contextual_variable_one_values":contextual_variable_one_values
        ,"contextual_variable_two_values":contextual_variable_two_values
        ,"contextual_variable_one_name":config_doc["contextual_variables"]["contextual_variable_one_name"]
        ,"contextual_variable_two_name":config_doc["contextual_variables"]["contextual_variable_two_name"]
        ,"contextual_variable_one_from_data_source":config_doc["contextual_variables"]["contextual_variable_one_from_data_source"]
        ,"contextual_variable_two_from_data_source":config_doc["contextual_variables"]["contextual_variable_two_from_data_source"]
        ,"contextual_variable_one_lookup":config_doc["contextual_variables"]["contextual_variable_one_lookup"]
        ,"contextual_variable_two_lookup":config_doc["contextual_variables"]["contextual_variable_two_lookup"]
        ,"offer_key":config_doc["contextual_variables"]["offer_key"]
        ,"tracking_key":config_doc["contextual_variables"]["tracking_key"]
        ,"options_store_database":config_doc["options_store_database"]
        ,"options_store_collection":config_doc["options_store_collection"]
        ,"prior_success_reward":config_doc["randomisation"]["prior_success_reward"]
        ,"prior_fail_reward":config_doc["randomisation"]["prior_fail_reward"]
        ,"take_up":config_doc["contextual_variables"]["take_up"]
    }
    cp.update_client_pulse_responder(auth, cp_update_doc)
    
    print("MESSAGE: Online learning configuration created")
    return config_doc["uuid"]


from setuptools import setup
from pathlib import Path

setup(
    name='ecosystem-notebooks',
    version='0.1.6',
    description='Ecosystem Notebooks is a wrapper for the Ecosystem API servers.',
    long_description=Path("README.rst").read_text(encoding="utf-8"),
    url='https://github.com/ecosystemai/ecosystem-notebooks',
    author='EcosystemAi',
    author_email='jay@ecosystem.ai',
    license='MIT',
    packages=['runtime', 'runtime.apis', 'runtime.endpoints', 'runtime.utils', 'prediction', 'prediction.apis', 'prediction.endpoints', 'prediction.utils'],
    # packages=['runtime', 'prediction', 'notebook_server'],
    # install_requires=['numpy',
				# 	'plotly',
				# 	'matplotlib',
				# 	'pymongo',
				# 	'gunicorn',
				# 	'python-dateutil',
				# 	'Flask',
				# 	'Flask-Compress',
				# 	'Flask-Cors',
				# 	'Flask-RESTful',
				# 	'flask-restplus',
				# 	'flask-restx',
				# 	'Flask-Session',
				# 	'jupyterhub-nativeauthenticator',
				# 	'psycopg2-binary',
				# 	'seaborn',
				# 	'requests',
				# 	'pandas',
				# 	'Werkzeug',
				# 	'prophet',
				# 	'networkx',
				# 	'torch>=1.11.0',
				# 	'transformers',
				# 	'accelerate>=0.20.1',
				# 	'ipywidgets==7.7.5',
				# 	'sentence-transformers',
				# 	'scikit-learn',
				# 	'statsmodels',
				# 	'h2o==3.42.0.2',
				# 	'nltk',
				# 	'wordcloud',
				# 	'shapely',
				# 	'faker',
				# 	'sdv',
				# 	'agentpy',
				# 	'langchain',
				# 	'openai',
				# 	'bs4',
				# 	'tiktoken',
				# 	'gpt4all',
				# 	'chromadb',
				# 	'jq',
				# 	'nest_asyncio',
				# 	'pypdf',
				# 	'chardet',
				# 	'statistics',
				# 	'datetime',
    #                   ],
    install_requires=[
					'requests',
					'pymongo'
                      ],

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',  
        'Operating System :: POSIX :: Linux',        
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.10',
        'Framework :: Jupyter :: JupyterLab'
    ],
)




from os import listdir
from os.path import isfile, join
import sys
import json

runtime_server_path = "C:/Users/Ramsay/Documents/GitHub/ecosystem-runtime/"
runtime_fp = runtime_server_path + "src/main/java/com/ecosystem/runtime/"
files = [f for f in listdir(runtime_fp) if isfile(join(runtime_fp, f))]

server_endpoints = []
api_request = "@RequestMapping"
api_post_request = "@PostMapping"

def stripping(parameter):
	if parameter[0] == "=":
		parameter = parameter[1:]
	parameter = parameter.replace("'", "")
	parameter = parameter.replace('"', "")
	if parameter[0] == ":":
		parameter = parameter[1:]
	# if parameter[0] == '"' or parameter[0] == "'":
	# 	parameter = parameter[1:]
	# if parameter[-1] == '"' or parameter[-1] == "'":	
	# 	parameter = parameter[:-1]
	return parameter

def create_endpoint(endpoint, method):
	d = {
		"endpoint": endpoint,
		"method": method
	}
	return d

def compare_endpoints(server_endpoints, wrapper_endpoints):
	both = []
	server = []
	wrapper = []
	for d in server_endpoints:
		se = d["endpoint"]
		sm = d["method"]
		found = False
		for d2 in wrapper_endpoints:
			we = d2["endpoint"]
			wm = d2["method"]
			if se == we and wm.lower() in sm.lower():
				both.append([d, d2])
				found = True
				break
		if not found:
			server.append(d)
	for d2 in wrapper_endpoints:
		we = d2["endpoint"]
		wm = d2["method"]
		found = False
		for d in server_endpoints:
			se = d["endpoint"]
			sm = d["method"]
			if se == we and wm.lower() in sm.lower():
				found = True
				break
		if not found:
			wrapper.append(d2)

	print("")
	print("Found in both:")
	data = [
		["----SERVER----", "", "", "----WRAPPER----", ""],
		["method", "endpoint", "", "method", "endpoint"],
		["------", "------", "------", "------", "------"]
	]
	for b in both:
		row = [b[0]["method"], b[0]["endpoint"], "|", b[1]["method"], b[1]["endpoint"]]
		data.append(row)
	col_width = max(len(word) for row in data for word in row) + 2
	for row in data:
		print("".join(word.ljust(col_width) for word in row))
	print("Count:{}".format(len(both)))

	print("Only in one:")
	data = [
		["----SERVER----", ""],
		["method", "endpoint"],
		["------", "------"]
	]
	for s in server:
		row = [s["method"], s["endpoint"]]
		data.append(row)
	col_width = max(len(word) for row in data for word in row) + 2
	for row in data:
		print("".join(word.ljust(col_width) for word in row))
	print("Count:{}".format(len(server)))
	data = [
		["----WRAPPER----", ""],
		["method", "endpoint"],
		["------", "------"]
	]
	for w in wrapper:
		row = [w["method"], w["endpoint"]]
		data.append(row)
	col_width = max(len(word) for row in data for word in row) + 2
	for row in data:
		print("".join(word.ljust(col_width) for word in row))
	print("Count:{}".format(len(wrapper)))

for f_n in files:
	file_path = runtime_fp + f_n
	f = open(file_path, "r")
	for line in f:
		if api_request in line:
			index = line.find(api_request)
			comment_index = line.find("//")
			if index != -1:
				if comment_index != -1:
					if comment_index < index:
						continue
				value = ""
				method = ""
				parameters = line[index+len(api_request):]
				parameters = parameters.split(",")
				for parameter in parameters:
					parameter = "".join(parameter.split())
					if parameter[0] == "(":
						parameter = parameter[1:]
					if parameter[-1] == ")":
						parameter = parameter[:-1]
					if "value" in parameter:
						index = parameter.find("value")
						parameter = parameter[index+len("value"):]
						value = stripping(parameter)

					elif "method" in parameter:
						index = parameter.find("method")
						parameter = parameter[index+len("method"):]
						method = stripping(parameter)
						
				server_endpoints.append(create_endpoint(value, method))
		if api_post_request in line:
			index = line.find(api_post_request)
			line = line[index+len(api_post_request):]
			line = line[2:-3]
			server_endpoints.append(create_endpoint(line, "POST"))


# for se in server_endpoints:
# 	print(se)

# print("count: {}".format(len(server_endpoints)))


wrapper_endpoints = []

wrapper_path = "../"
wrapper_fp = wrapper_path + "endpoints/"
files = [f for f in listdir(wrapper_fp) if isfile(join(wrapper_fp, f))]
if "__init__.py" in files:
	files.remove("__init__.py")

for f_n in files:
	file_path = wrapper_fp + f_n
	f = open(file_path, "r")
	new_data = []
	for line in f:
		line = "".join(line.split())
		if len(line) > 0:
			if line[0] != "#":
				new_data.append(line)
	data = "".join(new_data)
	while True:
		index = data.find("{")
		if index == -1:
			break
		open_count = 0
		close_count = 0
		for i in range(len(data[index:])):
			c = data[i+index]
			if c == "}":
				close_count += 1
			if c == "{":
				open_count += 1
			if open_count == close_count:
				s = data[index: index+i+1]
				j_str = eval(s)
				# j_str = json.loads(s)
				wrapper_endpoints.append(create_endpoint(j_str["endpoint"], j_str["type"]))
				data = data[index+i:]
				break

# for we in wrapper_endpoints:
# 	print(we)

# print("count: {}".format(len(wrapper_endpoints)))

compare_endpoints(server_endpoints, wrapper_endpoints)


from prediction.endpoints import ecosystem_main as endpoints
from prediction import request_utils

def create_profile(auth, json, info=False):
	ep = endpoints.CREATE
	resp = request_utils.create(auth, ep, json=json, info=info)
	result = resp.json()
	return result

def profiles(auth, info=False):
	ep = endpoints.PROFILES
	resp = request_utils.create(auth, ep, info=info)
	result = resp.json()
	return result


from prediction.endpoints import data_munging_engine as endpoints
from prediction import request_utils
# from prediction.apis import quickflat as qf

def concat_columns2(auth, database, collection, attribute, separator, info=False):
	ep = endpoints.CONCAT_COLUMNS2
	param_dict = {
		"mongodb": database, 
		"collection": collection,
		"attribute": attribute,
		"separator": separator
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def enrich_date2(auth, database, collection, attribute, find, info=False):
	ep = endpoints.DATE_ENRICH2
	param_dict = {
		"mongodb": database, 
		"collection": collection,
		"attribute": attribute,
		"find": find
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def auto_normalize_all(auth, database, collection, fields, find, normalized_high, normalized_low, info=False):
	ep = endpoints.AUTO_NORMALIZE_ALL
	param_dict = {
		"mongodb": database, 
		"collection": collection,
		"fields": fields,
		"find": find,
		"normalizedHigh": normalized_high,
		"normalizedLow": normalized_low
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def concat_columns(auth, databasename, collection, attribute, info=False):
	ep = endpoints.CONCAT_COLUMNS
	param_dict = {"mongodb": databasename, "collection": collection, "attribute": attribute}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def enrich_date(auth, database, collection, attribute, info=False):
	ep = endpoints.DATE_ENRICH
	param_dict = {
		"mongodb": database, 
		"collection": collection,
		"attribute": attribute
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def enum_convert(auth, database, collection, attribute, info=False):
	ep = endpoints.ENUM_CONVERT
	param_dict = {
		"mongodb": database, 
		"collection": collection,
		"attribute": attribute
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def fill_zeros(auth, database, collection, attribute, info=False):
	ep = endpoints.FILL_ZEROS
	param_dict = {
		"mongodb": database, 
		"collection": collection,
		"attribute": attribute
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def fill_values(auth, database, collection, find, attribute, value, info=False):
	ep = endpoints.FILL_VALUES
	param_dict = {
		"mongodb": database, 
		"collection": collection,
		"find": find,
		"attribute": attribute,
		"value": value
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta


def foreign_key_aggregator(auth, database, collection, attribute, search, mongodbf, collectionf, attributef, fields, info=False):
	ep = endpoints.FOREIGN_KEY_AGGREGATOR
	param_dict = {
		"mongodb": database,
		"collection": collection,
		"attribute": attribute,
		"search": search,
		"mongodbf": mongodbf,
		"collectionf": collectionf,
		"attributef": attributef,
		"fields": fields
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def foreign_key_lookup(auth, database, collection, attribute, search, mongodbf, collectionf, attributef, fields, info=False):
	ep = endpoints.FOREIGN_KEY_LOOKUP
	param_dict = {
		"mongodb": database,
		"collection": collection,
		"attribute": attribute,
		"search": search,
		"mongodbf": mongodbf,
		"collectionf": collectionf,
		"attributef": attributef,
		"fields": fields
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def enrich_fragments(auth, database, collection, attribute, strings, info=False):
	ep = endpoints.FRAGMENT_ENRICH
	param_dict = {
		"mongodb": database, 
		"collection": collection,
		"attribute": attribute,
		"stringOnly": strings
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def enrich_fragments2(auth, database, collection, attribute, strings, find, info=False):
	ep = endpoints.FRAGMENT_ENRICH2
	param_dict = {
		"mongodb": database, 
		"collection": collection,
		"attribute": attribute,
		"stringOnly": strings,
		"find": find
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def generate_features(auth, database, collection, featureset, categoryfield, datefield, numfield, groupby, find, info=False):
	ep = endpoints.GENERATE_FEATURES
	param_dict = {"database": database, "collection": collection, "featureset":featureset, "categoryfield":categoryfield, "datefield":datefield, "numfield":numfield, "groupby":groupby, "find":find}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def generate_features_normalize(auth, database, collection, find, inplace, normalized_high, normalized_low, numfields, info=False):
	ep = endpoints.GENERATE_FEATURES_NORMALIZE
	param_dict = {"database": database, "collection": collection, "find":find, "inPlace": inplace, "normalizedHigh": normalized_high, "normalizedLow": normalized_low, "numfields": numfields}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def get_categories(auth, database, collection, categoryfield, find, total, info=False):
	ep = endpoints.GET_CATEGORIES
	param_dict = {"database": database, "collection":collection, "categoryfield":categoryfield, "total": total, "find": find}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def get_categories_ratio(auth, database, collection, categoryfield, find, total, info=False):
	ep = endpoints.GET_CATEGORIES_RATIOS
	param_dict = {"database": database, "collection":collection, "categoryfield":categoryfield, "total": total, "find": find}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def enrich_location(auth, database, collection, attribute, info=False):
	ep = endpoints.LOCATION_ENRICH
	param_dict = {
		"mongodb": database, 
		"collection": collection,
		"attribute": attribute
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def enrich_mcc(auth, database, collection, attribute, find, info=False):
	ep = endpoints.MCC_ENRICH
	param_dict = {
		"mongodb": database, 
		"collection": collection,
		"attribute": attribute,
		"find": find
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def prediction_enrich_fast(auth, database, collection, search, sort, predictor, predictor_label, attributes, skip, limit, info=False):
	ep = endpoints.PREDICTION_ENRICH_FAST_GET
	param_dict = {
		"mongodb": database, 
		"collection": collection,
		"search": search,
		"sort": sort,
		"predictor": predictor,
		"predictor_label": predictor_label,
		"attributes": attributes,
		"skip": skip,
		"limit": limit
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta
	
# predicition_enrich(auth, database, collection2, search, sort, predictor, predictor_label, attributes) 
def predicition_enrich(auth, database, collection, search, sort, predictor, predictor_label, attributes, info=False):
	ep = endpoints.PREDICTION_ENRICH
	param_dict = {
		"mongodb": database, 
		"collection": collection,
		"search": search,
		"predictor": predictor,
		"predictor_label": predictor_label,
		"attributes": attributes,
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def enrich_sic(auth, database, collection, attribute, find, info=False):
	ep = endpoints.SIC_ENRICH
	param_dict = {
		"mongodb": database, 
		"collection": collection,
		"attribute": attribute,
		"find": find
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

# def quickflat(config, info=False):
# 	quick_flat = qf.QuickFlat(config)
# 	quick_flat.flatten()

def process_client_pulse_reliability(auth, collection, collectionOut, database, find, groupby, mongoAttribute, typeName, info=False):
	ep = endpoints.PROCESS_CLIENT_PULSE_RELIABILITY
	param_dict = {
		"collection": collection,
		"collectionOut": collectionOut,
		"database": database,
		"find": find,
		"groupby": groupby,
		"mongoAttribute": mongoAttribute,
		"type": typeName
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def generate_time_series_features(auth, categoryfield, collection, database, datefield, featureset, find, groupby, numfield, startdate=None, windowsize=1, info=False):
	ep = endpoints.GENERATE_TIME_SERIES_FEATURES
	param_dict = {
		"categoryfield": categoryfield,
		"collection": collection,
		"database": database,
		"datefield": datefield,
		"featureset": featureset,
		"find": find,
		"groupby": groupby,
		"numfield": numfield,
		"startdate": startdate,
		"windowsize": windowsize
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def personality_enrich(auth, category, collection, collectionOut, database, find, groupby, info=False):
	ep = endpoints.PERSONALITY_ENRICH
	param_dict = {
		"category": category,
		"collection": collection,
		"collectionOut": collectionOut,
		"database": database,
		"find": find,
		"groupby": groupby		
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def munge_transactions_aggregate(auth, munging_step, project_id, info=False):
	ep = endpoints.MUNGE_TRANSACTIONS_AGGREGATE
	param_dict = {
		"munging_step": munging_step,
		"project_id": project_id
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def munge_transactions(auth, munging_step, project_id, info=False):
	ep = endpoints.MUNGE_TRANSACTIONS
	param_dict = {
		"munging_step": munging_step,
		"project_id": project_id
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def flatten_document(auth, db, collection, attribute, find, info=False):
	ep = endpoints.FLATTEN_DOCUMENT
	param_dict = {
		"mongodb": db,
		"collection": collection,
		"attribute": attribute,
		"find": find
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def delete_key(auth, db, collection, attribute, find, info=False):
	ep = endpoints.DELETE_KEY
	param_dict = {
		"mongodb": db,
		"collection": collection,
		"attribute": attribute,
		"find": find
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def delete_many_documents(auth, db, collection, find, info=False):
	ep = endpoints.DELETE_MANY_DOCUMENTS
	param_dict = {
		"mongodb": db,
		"collection": collection,
		"find": find
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def process_range(auth, db, collection, attribute, new_attribute, find, rules, info=False):
	ep = endpoints.PROCESS_RANGE
	param_dict = {
		"database": db,
		"collection": collection,
		"mongoAttribute": attribute,
		"newAttribute": new_attribute,
		"find": find,
		"rules": rules
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def nlp_worker(auth, database, collection, database_out, collection_out, attribute, find, model="original", summarization_max=10, summarization_min=5, transformer="t5-small", model_type="nlp_b5_base", info=False):
	ep = endpoints.NLP_WORKER
	param_dict = {
		"database": db,
		"collection": collection,
		"database_out": database_out,
		"collection_out": collection_out,
		"attribute": attribute,
		"find": find,
		"model": model,
		"summarization_max": summarization_max,
		"summarization_min": summarization_min,
		"transformer": transformer,
		"type": model_type,
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def delete_many_documents(auth, db, collection, find, info=False):
	ep = endpoints.DELETE_MANY_DOCUMENTS
	param_dict = {
		"mongodb": db,
		"collection": collection,
		"find": find
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta

def process_range(auth, db, collection, find, attribute, new_attribute, rules, info=False):
	ep = endpoints.PROCESS_RANGE
	param_dict = {
		"database": db,
		"collection": collection,
		"find": find,
		"mongoAttribute": attribute,
		"newAttribute": new_attribute,
		"rules": rules
	}
	resp = request_utils.create(auth, ep, params=param_dict, info=info)
	meta = resp.json()
	return meta


def prediction_enrich_fast_post(auth, json, info=False):
	ep = endpoints.PREDICTION_ENRICH_FAST_POST
	resp = request_utils.create(auth, ep, json=json, info=info)
	result = resp.json()
	return result


UPDATE_PROPERTIES = {
	"type": "post",
	"endpoint": "/updateProperties",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

UPLOAD_FILE = {
	"type": "post",
	"endpoint": "/upload",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

GET_FILE = {
	"type": "get",
	"endpoint": "/getFile",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

GET_FILE_TAIL = {
	"type": "get",
	"endpoint": "/getFileTail",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

GET_PROPERTY = {
	"type": "get",
	"endpoint": "/getProperty",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

GET_FILES = {
	"type": "get",
	"endpoint": "/getFiles",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}

COPY_FILE = {
	"type": "get",
	"endpoint": "/copyFile",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

DOWNLOAD = {
	"type": "get",
	"endpoint": "/download",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

DELETE_FILE = {
	"type": "get",
	"endpoint": "/deleteFile",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

FILE_DELETE = {
	"type": "delete",
	"endpoint": "/file",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

CONVERT_REMOVE_NON_PRINTABLES = {
	"type": "get",
	"endpoint": "/convertRemoveNonPrintables",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

PROCESS_PUSH = {
	"type": "post",
	"endpoint": "/processPush",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

CONVERT_CSV_FROM_TO = {
	"type": "get",
	"endpoint": "/convertCsvFromTo",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

CONVERT_TEXT_FILE_FROM_TO = {
	"type": "get",
	"endpoint": "/convertTextFileFromTo",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}


from runtime.endpoints import predictor_engine as endpoints
from runtime import request_utils


def get_spending_personality(auth, campaign, channel, customer, headers, params, subcampaign, userid, info=False):
# Provide spending personality scores for customers.
#   auth: Authentication token generated by access.Authenticate()
#   campaign: The name of the campaign. (string)
#   channel: The type of channel. (options: "all", "") 
#   customer: The id of the customer. (string)
#   headers: Added headers. ()
#   params: Addtional parameters added in a dictionary format as a string: ('{"value_1": 300, "value_2": "entry"}')
#   subcampaign: Name of the subcampaigns. (string)
#   userid: The id of the user. (string)
    ep = endpoints.GET_SPENDING_PERSONALITY
    param_dict = {
        "campaign": campaign, 
        "channel": channel,
        "customer": customer,
        "headers": headers,
        "params": params,
        "subcampaign": subcampaign,
        "userid": userid
    }
    resp = request_utils.create(auth, ep, params=param_dict, info=info)
    meta = resp.json()
    return meta

def put_spending_personality(auth, document, headers, info=False):
# Update offers taken up by customers
#  document: Documents to be updated. ()
#  headers: Added headers. ()
    ep = endpoints.PUT_SPENDING_PERSONALITY
    param_dict = {
        "document": document, 
        "headers": headers
    }
    resp = request_utils.create(auth, ep, params=param_dict, info=info)
    meta = resp.json()
    return meta

def model_detail(auth, model, info=False):
# Model details. 
# model: Model's details to examine. Example parameter: {'mojo':'my_mojo.zip'}
    ep = endpoints.GET_OFFER_RECOMMENDATIONS
    param_dict = {
        "model": model
    }
    resp = request_utils.create(auth, ep, params=param_dict, info=info)
    meta = resp.json()
    return meta

def get_offer_recommendations(auth, campaign, channel, customer, headers, numberoffers, params, subcampaign, userid, info=False):
# Provide offers that form part of a campaign for particular customer.
#   auth: Authentication token generated by access.Authenticate()
#   campaign: The name of the campaign. (string)
#   channel: The type of channel. (options: "all", "") 
#   customer: The id of the customer. (string)
#   headers: Added headers. ()
#   numberoffers: Number of offers to get. (int)
#   params: Addtional parameters added in a dictionary format as a string: ('{"value_1": 300, "value_2": "entry"}')
#   subcampaign: Name of the subcampaigns. (string)
#   userid: The id of the user. (string)
    ep = endpoints.GET_OFFER_RECOMMENDATIONS
    param_dict = {
        "campaign": campaign, 
        "channel": channel,
        "customer": customer,
        "headers": headers,
        "numberoffers": numberoffers,
        "params": params,
        "subcampaign": subcampaign,
        "userid": userid
    }
    resp = request_utils.create(auth, ep, params=param_dict, info=info)
    meta = resp.json()
    return meta

def put_offer_recommendations(auth, document, headers, info=False):
# Update offers taken up by customers
#  document: Documents to be updated. ()
#  headers: Added headers. ()
    ep = endpoints.PUT_OFFER_RECOMMENDATIONS
    param_dict = {
        "document": document, 
        "headers": headers
    }
    resp = request_utils.create(auth, ep, params=param_dict, info=info)
    meta = resp.json()
    return meta

def get_personality_recommender(auth, campaign, channel, customer, headers, numberoffers, params, subcampaign, userid, info=False):
# Provide offers that form part of a campaign for particular customer.
#   auth: Authentication token generated by access.Authenticate()
#   campaign: The name of the campaign. (string)
#   channel: The type of channel. (options: "all", "") 
#   customer: The id of the customer. (string)
#   headers: Added headers. ()
#   numberoffers: Number of offers to get. (int)
#   params: Addtional parameters added in a dictionary format as a string: ('{"value_1": 300, "value_2": "entry"}')
#   subcampaign: Name of the subcampaigns. (string)
#   userid: The id of the user. (string)
    ep = endpoints.GET_PERSONALITY_RECOMMENDER
    param_dict = {
        "campaign": campaign, 
        "channel": channel,
        "customer": customer,
        "headers": headers,
        "numberoffers": numberoffers,
        "params": params,
        "subcampaign": subcampaign,
        "userid": userid
    }
    resp = request_utils.create(auth, ep, params=param_dict, info=info)
    meta = resp.json()
    return meta

def put_personality_recommender(auth, document, headers, info=False):
# Update offers taken up by customers
#  document: Documents to be updated. ()
#  headers: Added headers. ()
    ep = endpoints.PUT_PERSONALITY_RECOMMENDER
    param_dict = {
        "document": document, 
        "headers": headers
    }
    resp = request_utils.create(auth, ep, params=param_dict, info=info)
    meta = resp.json()
    return meta

def predictor_response_preload(auth, detail, value, info=False):
# Perform prediction on pre-loaded model with detail: none, basic or all. 
# Perform a database lookup if properties file has been set. 
# The predictor parameters are broken into two types namely, 
# requiring all parameters via API or requiring a lookup key via API and extracting parameters from a data source.
# Use this format for input prams only:
# Update offers taken up by customers
#  detail: Documents to be updated. (string: "none", "basic", "all")
#  value: Input parameter. 
#  {
#   'name':'predict1', 
#   'mojo':'model_mojo.zip',
#   'dbparam':false,
#   'input': ['x','y'],
#   'value': ['val_x', 'val_y']
#  }
# Use this approach for inputs from data source: 
#  {
#   'name':'predict1', 
#   'mojo':'model_mojo.zip',
#   'dbparam':true, 
#   'lookup':{key:'customer',value:1234567890}
#  } 
# For post-scoring logic, then use this configuration:
#  {
#   'name':'predict1', 
#   'mojo':'1',
#   'mab':{'class':'mabone', 'epsilon':0.4},
#   'dbparam':true, 
#   'lookup':{key:'customer',value:1234567890}, 
#   'param':{key:'value_field', value:30}
#}
    ep = endpoints.PREDICTOR_RESPONSE_PRELOAD
    param_dict = {
        "detail": detail, 
        "value": value
    }
    resp = request_utils.create(auth, ep, params=param_dict, info=info)
    meta = resp.json()
    return meta

def predictor_response_preload_kafka(auth, detail, value, info=False):
# Perform prediction on pre-loaded model with detail and push onto Kafka topic: none, basic or all.
# Perform a database lookup if properties file has been set. 
#  detail: Documents to be updated. (string: "none", "basic", "all")
#  value: Input parameter. 
#  {
#    'name':'predict1', 
#    'kafka':{'TOPIC_NAME':'ecosystem1','log':'true'},
#    'mojo':'1', 'input':['x','y'], 
#    'value':['val_x','val_y']
#  } 
#  OR 
#  {
#    'name':'predict1',
#    'kafka':{'TOPIC_NAME':'ecosystem1','log':'true'},
#    'mojo':'1',
#    'dbparam':true,
#    'lookup':{key:'customer',value:'1234567890'} 
#  }
    ep = endpoints.PREDICTOR_RESPONSE_PRELOAD_KAFKA
    param_dict = {
        "detail": detail, 
        "value": value
    }
    resp = request_utils.create(auth, ep, params=param_dict, info=info)
    meta = resp.json()
    return meta

def refresh(auth, headers, info=False):
# Refresh product matrix and master
#  headers: Added headers. ()
    ep = endpoints.REFRESH
    param_dict = {
        "headers": headers
    }
    resp = request_utils.create(auth, ep, params=param_dict, info=info)
    meta = resp.json()
    return meta

def run_model_mojo(auth, detail, value, info=False):
# Perform basic prediction on model with detail: none, basic or all. 
#  detail: Documents to be updated. (string: "none", "basic", "all")
#  value: Input parameter.
#  {
#    'mojo':'model_mojo.zip',
#    'input': ['x','y'],
#    'value': ['val_x', 'val_y']
#  }
    ep = endpoints.RUN_MODEL_MOJO
    param_dict = {
        "detail": detail, 
        "value": value
    }
    resp = request_utils.create(auth, ep, params=param_dict, info=info)
    meta = resp.json()
    return meta

def invocations(auth, json, info=False):
    ep = endpoints.INVOCATIONS
    resp = request_utils.create(auth, ep, json=json, info=info)
    print(resp)
    meta = resp.json()
    return meta

def response(auth, json, info=False):
    ep = endpoints.RESPONSE
    resp = request_utils.create(auth, ep, json=json, info=info)
    meta = resp.json()
    return meta

def process_batch(auth, json, info=False):
    ep = endpoints.PROCESS_BATCH
    resp = request_utils.create(auth, ep, json=json, info=info)
    meta = resp.json()
    return meta

def generate_key(auth, info=False):
    ep = endpoints.GENERATE_KEY
    resp = request_utils.create(auth, ep, info=info)
    meta = resp.json()
    return meta